@@ -1,6 +1,3 @@
-## Please see file perltidy.ERR
-## Please see file perltidy.ERR
-## Please see file perltidy.ERR
 package App::SpamcupNG;
 use warnings;
 use strict;
@@ -8,18 +5,23 @@ use LWP::UserAgent 6.05;
 use HTML::Form 6.07;
 use HTTP::Cookies 6.01;
 use Getopt::Std;
-use HTML::Entities 3.76;
 use YAML::XS 0.62 qw(LoadFile);
 use File::Spec;
 use Hash::Util qw(lock_hash);
 use Exporter 'import';
 use Log::Log4perl 1.48 qw(get_logger :levels);
 use Carp;
-use HTML::TreeBuilder::XPath 0.14;
+
+use App::SpamcupNG::HTMLParse (
+    'find_next_id',       'find_errors',
+    'find_warnings',      'find_spam_header',
+    'find_best_contacts', 'find_receivers'
+);
+
 use constant TARGET_HTML_FORM => 'sendreport';
 
-our @EXPORT_OK =
-  qw(read_config main_loop get_browser %OPTIONS_MAP config_logger TARGET_HTML_FORM);
+our @EXPORT_OK
+    = qw(read_config main_loop get_browser %OPTIONS_MAP config_logger TARGET_HTML_FORM);
 our %OPTIONS_MAP = (
     'check_only' => 'n',
     'all'        => 'a',
@@ -42,7 +44,7 @@ my %regexes = (
 
 lock_hash(%OPTIONS_MAP);
 
-our $VERSION = '0.003'; # VERSION
+our $VERSION = '0.004'; # VERSION
 
 =head1 NAME
 
@@ -87,8 +89,8 @@ sub read_config {
     # sanity checking
     for my $opt ( keys( %{ $data->{ExecutionOptions} } ) ) {
         die
-"$opt is not a valid option for configuration files. Check the documentation."
-          unless ( exists( $OPTIONS_MAP{$opt} ) );
+            "$opt is not a valid option for configuration files. Check the documentation."
+            unless ( exists( $OPTIONS_MAP{$opt} ) );
     }
 
     for my $opt ( keys(%OPTIONS_MAP) ) {
@@ -207,9 +209,9 @@ L<Log::Log4perl> for more details about the levels.
 sub config_logger {
     my ( $level, $log_file ) = @_;
     croak "Must receive a string for the level parameter"
-      unless ( ( defined($level) ) and ( $level ne '' ) );
+        unless ( ( defined($level) ) and ( $level ne '' ) );
     croak "Must receive a string for the log file parameter"
-      unless ( ( defined($log_file) ) and ( $log_file ne '' ) );
+        unless ( ( defined($log_file) ) and ( $log_file ne '' ) );
 
 # :TODO:21/01/2018 12:07:01:ARFREITAS: Do we need to import :levels from Log::Log4perl at all?
     my %levels = (
@@ -220,7 +222,7 @@ sub config_logger {
         FATAL => $FATAL
     );
     croak "The value '$level' is not a valid value for level"
-      unless ( exists( $levels{$level} ) );
+        unless ( exists( $levels{$level} ) );
 
     my $conf;
 
@@ -336,7 +338,7 @@ sub _self_auth {
         else {
             $logger->warn($res_status);
             $logger->fatal(
-'Cannot connect to server or invalid credentials. Please verify your username and password and try again.'
+                'Cannot connect to server or invalid credentials. Please verify your username and password and try again.'
             );
         }
     }
@@ -346,7 +348,7 @@ sub _self_auth {
     # Parse id for link
     if ( $content =~ $regexes{no_user_id} ) {
         $logger->logdie(
-'No userid found. Please check that you have entered correct code. Also consider obtaining a password to Spamcop.net instead of using the old-style authorization token.'
+            'No userid found. Please check that you have entered correct code. Also consider obtaining a password to Spamcop.net instead of using the old-style authorization token.'
         );
     }
 
@@ -359,101 +361,6 @@ sub _self_auth {
 
 }
 
-sub _check_next_id {
-    my $content_ref = shift;
-    my $next_id;
-    my $logger = get_logger('SpamcupNG');
-
-    if ( $$content_ref =~ $regexes{next_id} ) {
-        $next_id = $1;
-        $logger->info("ID of the next SPAM is '$next_id'");
-    }
-    else {
-        # userid ok, no new spam
-        $logger->info('No unreported SPAM found.');
-    }
-
-    return $next_id;
-}
-
-# TODO: create a single tree instance and check for everything at once
-sub _check_warning {
-    my $content_ref = shift;
-    my $tree        = HTML::TreeBuilder::XPath->new;
-    $tree->parse_content($$content_ref);
-    my @errors = $tree->findnodes('//div[@id="content"]/div[@class="warn"]');
-
-    if ( scalar(@errors) > 0 ) {
-        return $errors[0]->as_trimmed_text;
-    }
-    else {
-        return;
-    }
-}
-
-sub _check_error {
-    my $content_ref = shift;
-    my $tree        = HTML::TreeBuilder::XPath->new;
-    $tree->parse_content($$content_ref);
-    my @errors = $tree->findnodes('//div[@id="content"]/div[@class="error"]');
-
-    if ( scalar(@errors) > 0 ) {
-        return $errors[0]->as_trimmed_text;
-    }
-    else {
-        return;
-    }
-}
-
-sub _find_best_contacts {
-    my $content_ref = shift;
-    my $tree        = HTML::TreeBuilder::XPath->new;
-    $tree->parse_content($content_ref);
-    my @nodes = $tree->findnodes('//div[@id="content"]');
-
-    foreach my $node (@nodes) {
-        for my $html_element ( $node->content_list ) {
-
-            # only text
-            next if ref($html_element);
-            $html_element =~ s/^\s+//;
-            if ( index( $html_element, 'Using best contacts' ) == 0 ) {
-                my @tokens = split( /\s/, $html_element );
-                splice( @tokens, 0, 3 );
-                return \@tokens;
-            }
-        }
-
-    }
-
-    return [];
-}
-
-# TODO: parse this without regex
-sub _spam_header {
-    my $raw_spam_header = shift;
-    my $formatted //= 0;
-    my $tree = HTML::TreeBuilder::XPath->new;
-    $tree->parse_content($raw_spam_header);
-    my @nodes = $tree->findnodes_as_strings('//text()');
-    my @lines;
-
-    for ( my $i = 0 ; $i <= scalar(@nodes) ; $i++ ) {
-        next unless $nodes[$i];
-        $nodes[$i] =~ s/^\s++//u;
-
-        if ($formatted) {
-            push( @lines, "\t$nodes[$i]" );
-
-        }
-        else {
-            push( @lines, $nodes[$i] );
-
-        }
-    }
-    return \@lines;
-}
-
 sub main_loop {
     my ( $ua, $opts_ref ) = @_;
     my $logger = get_logger('SpamcupNG');
@@ -473,7 +380,7 @@ sub main_loop {
     my $next_id;
 
     if ($response) {
-        $next_id = _check_next_id( \$response );
+        $next_id = find_next_id( \$response );
         return -1 unless ( defined($next_id) );
     }
     else {
@@ -483,7 +390,7 @@ sub main_loop {
     # avoid loops
     if ( ($last_seen) and ( $next_id eq $last_seen ) ) {
         $logger->fatal(
-"I have seen this ID earlier, we do not want to report it again. This usually happens because of a bug in Spamcup. Make sure you use latest version! You may also want to go check from Spamcop what is happening: http://www.spamcop.net/sc?id=$next_id"
+            "I have seen this ID earlier, we do not want to report it again. This usually happens because of a bug in Spamcup. Make sure you use latest version! You may also want to go check from Spamcop what is happening: http://www.spamcop.net/sc?id=$next_id"
         );
     }
 
@@ -497,8 +404,8 @@ sub main_loop {
     sleep $opts_ref->{delay};
 
     # Getting a SPAM report
-    my $req =
-      HTTP::Request->new( GET => 'http://www.spamcop.net/sc?id=' . $next_id );
+    my $req = HTTP::Request->new(
+        GET => 'http://www.spamcop.net/sc?id=' . $next_id );
 
     if ( $logger->is_debug ) {
         $logger->debug( "Request to be sent:\n" . $req->as_string );
@@ -515,32 +422,38 @@ sub main_loop {
         return 0;
     }
 
-    if ( my $warn_msg = _check_warning( \( $res->content ) ) ) {
-        $logger->warn($warn_msg);
+    if ( my $warns_ref = find_warnings( \( $res->content ) ) ) {
+        foreach my $warning ( @{$warns_ref} ) {
+            $logger->warn($warning);
+        }
     }
 
-    if ( my $error_msg = _check_error( \( $res->content ) ) ) {
+    if ( my $errors_ref = find_errors( \( $res->content ) ) ) {
 
-        my $is_fatal = 0;
+        foreach my $error ( @{$errors_ref} ) {
+            my $is_fatal = 0;
 
-        for my $fatal_error ( keys(%fatal_errors) ) {
+            foreach my $fatal_error ( keys(%fatal_errors) ) {
 
-            if ( $error_msg =~ $fatal_errors{$fatal_error} ) {
-                $is_fatal = 1;
-                last;
+                if ( $error =~ $fatal_errors{$fatal_error} ) {
+                    $is_fatal = 1;
+                    last;
+                }
+
+            }
+
+            if ($is_fatal) {
+                $logger->fatal($error);
+
+              # must stop processing the HTML for this report and move to next
+                return 0;
+            }
+            else {
+                $logger->error($error);
             }
 
         }
 
-        if ($is_fatal) {
-            $logger->fatal($error_msg);
-
-            # must stop processing the HTML for this report and move to next
-            return 0;
-        }
-        else {
-            $logger->error($error_msg);
-        }
     }
 
     # parsing the SPAM
@@ -549,7 +462,7 @@ sub main_loop {
 
     unless ($base_uri) {
         $logger->fatal(
-'No base URI found. Internal error? Please report this error by registering an issue on Github'
+            'No base URI found. Internal error? Please report this error by registering an issue on Github'
         );
     }
 
@@ -558,20 +471,20 @@ sub main_loop {
     }
 
     if ( $logger->is_info ) {
-        my $best_ref     = _find_best_contacts( $res->content );
+        my $best_ref     = find_best_contacts( $res->content );
         my $best_as_text = join( ', ', @$best_ref );
         $logger->info("Best contacts for SPAM reporting: $best_as_text");
     }
 
     my $form = _report_form( $res->content, $base_uri );
 
-    if ( $res->content =~
-/Please make sure this email IS spam.*?size=2\>\n(.*?)\<a href\=\"\/sc\?id\=$next_id/sgi
-      )
+    if ( $res->content
+        =~ /Please make sure this email IS spam.*?size=2\>\n(.*?)\<a href\=\"\/sc\?id\=$next_id/sgi
+        )
     {
 
         if ( $logger->is_info ) {
-            my $spam_header_ref = _spam_header($1);
+            my $spam_header_ref = find_spam_header($1);
             my $as_string       = join( "\n", @$spam_header_ref );
             $logger->info("Head of the SPAM follows:\n$as_string");
         }
@@ -581,7 +494,7 @@ sub main_loop {
         unless ($form) {
 
             $logger->fatal(
-'Could not find the HTML form to report the SPAM! May be a temporary Spamcop.net error, try again later! Quitting...'
+                'Could not find the HTML form to report the SPAM! May be a temporary Spamcop.net error, try again later! Quitting...'
             );
         }
         else {
@@ -596,7 +509,7 @@ sub main_loop {
             my $wontsend;
 
             # iterate targets
-            for ( my $i = 1 ; $i <= $max ; $i++ ) {
+            for ( my $i = 1; $i <= $max; $i++ ) {
                 my $send   = $form->value("send$i");
                 my $type   = $form->value("type$i");
                 my $master = $form->value("master$i");
@@ -607,11 +520,10 @@ sub main_loop {
                     $info =~ s/%([A-Fa-f\d]{2})/chr hex $1/eg;
                 }
 
-                if (
-                    $send
+                if ($send
                     and (  ( $send eq 'on' )
                         or ( $type =~ /^mole/ and $send == 1 ) )
-                  )
+                    )
                 {
                     $willsend .= "\t$master \t($info)\n";
                 }
@@ -622,8 +534,8 @@ sub main_loop {
 
             if ( $logger->is_info ) {
 
-                my $message =
-'Would send the report to the following addresses (reason in parenthesis): ';
+                my $message
+                    = 'Would send the report to the following addresses (reason in parenthesis): ';
 
                 if ($willsend) {
                     $message .= $willsend;
@@ -676,7 +588,7 @@ sub main_loop {
 
         unless ( $opts_ref->{stupid} ) {
             print
-"* Preview headers not available, but you can still report this. Are you sure this is spam? [y/N] ";
+                "* Preview headers not available, but you can still report this. Are you sure this is spam? [y/N] ";
 
             my $reply = <>;
             chomp($reply);
@@ -694,14 +606,14 @@ sub main_loop {
         }
 
     }
-    elsif ( $res->content =~
-/click reload if this page does not refresh automatically in \n(\d+) seconds/gs
-      )
+    elsif ( $res->content
+        =~ /click reload if this page does not refresh automatically in \n(\d+) seconds/gs
+        )
 
     {
         my $delay = $1;
         $logger->warn(
-"Spamcop seems to be currently overloaded. Trying again in $delay seconds. Wait..."
+            "Spamcop seems to be currently overloaded. Trying again in $delay seconds. Wait..."
         );
         sleep $opts_ref->{delay};
 
@@ -711,25 +623,25 @@ sub main_loop {
         # fake that everything is ok
         return 1;
     }
-    elsif ( $res->content =~
-        /No source IP address found, cannot proceed. Not full header/gs )
+    elsif ( $res->content
+        =~ /No source IP address found, cannot proceed. Not full header/gs )
     {
         $logger->warn(
-'No source IP address found. Your report might be missing headers. Skipping.'
+            'No source IP address found. Your report might be missing headers. Skipping.'
         );
         return 0;
     }
     else {
-        # Shit happens. If you know it should be parseable, please report a bug!
+      # Shit happens. If you know it should be parseable, please report a bug!
         $logger->warn(
-"Can't parse Spamcop.net's HTML. If this does not happen very often you can ignore this warning. Otherwise check if there's new version available. Skipping."
+            "Can't parse Spamcop.net's HTML. If this does not happen very often you can ignore this warning. Otherwise check if there's new version available. Skipping."
         );
         return 0;
     }
 
     if ( $opts_ref->{check_only} ) {
         $logger->info(
-'You gave option -n, so we\'ll stop here. The SPAM was NOT reported.'
+            'You gave option -n, so we\'ll stop here. The SPAM was NOT reported.'
         );
         exit;
     }
@@ -748,12 +660,12 @@ sub main_loop {
         }
         sleep $opts_ref->{delay};
         $res = LWP::UserAgent->new->request( $form->click() )
-          ;    # click default button, submit
+            ;    # click default button, submit
     }
-    else {     # CANCEL SPAM
+    else {       # CANCEL SPAM
         $logger->debug('About to cancel report.');
         $res = LWP::UserAgent->new->request( $form->click('cancel') )
-          ;    # click cancel button
+            ;    # click cancel button
     }
 
     if ( $logger->is_debug ) {
@@ -762,45 +674,41 @@ sub main_loop {
 
     # Check the outcome of the response
     unless ( $res->is_success ) {
-        $logger->fatal('Cannot connect to server. Try again later. Quitting.');
+        $logger->fatal(
+            'Cannot connect to server. Try again later. Quitting.');
         return 0;
     }
 
     if ($_cancel) {
-        return 1;    # user decided this mail is not spam
+        return 1;    # user decided this mail is not SPAM
     }
 
-    # parse respond
-    my $report;
+    # parse response
+    my $receivers_ref = find_receivers( $res->content );
 
-    if ( $res->content =~ /(Spam report id .*?)\<p\>/gsi ) {
-        $report = $1 || "-none-\n";
-        $report =~ s/\<br\>//gi;
-    }
-    elsif ( $res->content =~ /report for mole\@devnull.spamcop.net/ ) {
-        $report = 'Mole report(s)';
-    }
-    elsif ( $res->content =~ /\/dev\/null/ ) {
-        my $tree = HTML::TreeBuilder::XPath->new;
-        $tree->parse_content( $res->content );
-        my @dev_nulling = $tree->findnodes('//*[@id="content"]');
-        $report = $dev_nulling[0]->as_trimmed_text;
-    }
-    else {
-        $logger->warn(
-'Spamcop.net returned unexpected content (no SPAM report id). If this does not happen very often you can ignore this. Otherwise check if there new version available. Continuing.'
-        );
-    }
+    if ( scalar( @{$receivers_ref} ) > 0 ) {
 
-    # print the report
-    if ( $logger->is_info ) {
+        # print the report
+        if ( $logger->is_info ) {
 
-        if ($report) {
-            $logger->info("Spamcop.net sent following SPAM reports:\n$report");
+            my $report = join( "\n", @{$receivers_ref} );
+            $logger->info(
+                "Spamcop.net sent following SPAM reports:\n$report");
+
+            $logger->info('Finished processing.');
         }
 
-        $logger->info('Finished processing.');
-
+    }
+    else {
+        my $msg = <<'EOM';
+Spamcop.net returned unexpected content (no SPAM report id, no receiver).
+Please make check if there new version of App-SpamcupNG available and upgrade it.
+If you already have the latest version, please open a bug report in the
+App-SpamcupNG homepage and provide the next lines with the HTML response
+provided by Spamcop.
+EOM
+        $logger->warn($msg);
+        $logger->warn( $res->content );
     }
 
     return 1;
